/*  Copyright (c) 2009 Delft University of Technology

    This file is part of the mac-framework package.

    The mac-framework pakcage is free software: you can redistribute it
    and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    The mac-framework pakcage is distributed in the hope that it will be
    useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the mac-framework package.  If not, see
    <http://www.gnu.org/licenses/>.
*/

#ifndef TIMEFUNCS_H
#define TIMEFUNCS_H

#define GT_OP_SM(a,b,op) (a.high != 0 ? a.high op 0 : a.low op b)
#define GT_OP(a,b,op) (a.high != b.high ? a.high op b.high : a.low op b.low)
#define GT_GT(a,b) GT_OP(a,b,>)
#define GT_GT_SM(a,b) GT_OP_SM(a,b,>)
#define GT_LT(a,b) GT_OP(a,b,<)
#define GT_EQ(a,b) (a.high == b.high ? a.low == b.low : FALSE)
#define GT_NE(a,b) (a.high != b.high ? TRUE : a.low != b.low)

#define GT_ADD(a,b,out) do {uint32_t tf = (a).low+(b).low;(out).high = (a).high+(b).high; if (tf<(a).low||tf<(b).low){(out).high ++;}; (out).low=tf;} while(0)
#define GT_ADD_SM(a,small,out) do {uint32_t tf = (a).low+(small);(out).high = (a).high; if (tf<(a).low||tf<(small)){(out).high ++;}; (out).low=tf;} while(0)
#define GT_SUB(a,b,out) do {(out).high = (a).high-(b).high;if ((a).low<(b).low){(out).high--;} (out).low = (a).low-(b).low;} while(0)
#define GT_SUB_SM(a,small,out) do {(out).high = (a).high;if ((a).low<small){(out).high--;} (out).low = (a).low-(small);} while(0)
#define GT_MOD(a,small) mod_gt(a,small)

#define GT_COPY(in,out) do {(out).high = (in).high;(out).low=(in).low;} while(0)
#define GT_INIT(out,small) do{(out).low = small;(out).high=0;}while(0)

#define GT_TOP(a,bits) ((a)>>(32-bits))
#define GT_LOW_TOP(a,bits) GT_TOP((a).low,bits)

#define GT_MULT2(in,out,count) do {(out).high = ((in).high<<count)|GT_LOW_TOP((in),count);(out).low = (in).low << count;} while(0)
#define GT_DIV2(in,out,count) do {(out).low = (in).low >> count|(((uint32_t)(in).high)<<(32-count));(out).high = (in).high>>count;} while(0)

inline uint32_t mod_gt(globaltime_t a, uint32_t b)
{
	if (a.high!=0)
	{
		globaltime_t temp;
		GT_INIT(temp,b);
		while (GT_GT(a,temp) && !(temp.high & 0x80))
		{
			GT_MULT2(temp,temp,1);
		}

		while (a.high > 0)
		{
			while(GT_GT(temp,a))
				GT_DIV2(temp,temp,1);

			GT_SUB(a,temp,a);
		}
	}

	return a.low%b;
}

#endif
